;==============================================================================
;-------------------------------
;Read S-records and Burn Routine
;-------------------------------
Burn_s_records
	clr	ERROR				;no errors yet

Get_S
	jsr	GetChar				;look for S record
	cmpa	#'S
	bne	Get_S

	jsr	GetChar
	cmpa	#S1REcType			;if not S1 record then
	lbeq	Load1

	cmpa	#S2REcType			;if S2 record then
	beq	Load2

	cmpa	#S8REcType
	beq	S2_end

	cmpa	#S9REcType			;if S9 record then
	bne	Get_S
		
	jsr	GetHexByte			;get byte count
	suba    #2
        staa    byte_counter
	jsr	GetAddress			;get execution address

Load9 
	jsr	GetHexByte			;get checksum byte (and ignore it)
        dec     byte_counter
	bne	Load9
	lbra	PSX

S2_end
	jsr	GetHexByte		;get byte count
	staa	byte_counter
	
check_s2_end
	jsr	GetHexByte
	dec	byte_counter
	bne	check_s2_end
	lbra	PSX

Load2
	movb	#1,s_record_type	;Set for S2 record

	jsr	GetHexByte		;S2 record, so get byte count
	suba	#4			;adjust count for (24bit)address and checksum bytes
	staa    byte_counter		;save record lenght

	jsr	GetHexByte		;get upper part of address (A21-A16)
	psha				; save top of address
	jsr	GetAddress		;get lower part of address (A15-A0)
	sty	S_ADR

	xgdy

	psha				; save upper part of address range (A15-A12)
	anda	#$3F			; mask off A15 and A14 this allows to place the address 
					; into range from $8000 to $bfff
	addd	#$8000			; add Flash ROM base address offset $8000+(A13-A00)

	xgdy				;Reg Y contains the physical address into the page ($8000 to $Bfff)

	tst	BANKING
	beq	banking_mode

	pulb				; get (A15-A12)
	pula				; get (A21-A16)

	lsld				; A14 is a 16K boundary and must be shifted twice
	lsld				; 

	staa	PPAGE			;this allow to select the page (A21-A14)
	bra	check_even_odd_address

banking_mode
	pulb				; get (A15-A12)
	pula				; get (A21-A16)

	ldx	S_ADR
	cpx	#$C000
	bhs	last_page

	movb	LAST_PPAGE,PPAGE		;$C000 to $FFFF is last page
	dec	PPAGE				;decrement PPAGE so that range is $8000 to $BFFF

	bra	check_even_odd_address

last_page
	movb	LAST_PPAGE,PPAGE		;$C000 to $FFFF is last page
	bra	check_even_odd_address

Load1 
	clr	s_record_type

	jsr	GetHexByte		;S1 record, so get byte count
        suba    #3
        staa    byte_counter		;save record lenght

	jsr	GetAddress		;get address

check_even_odd_address
	pshy
        puld
        lsrd				;check for odd address
        bcs     set_odd_address
        bra     set_normal_address

set_odd_address 
	lsld
        movb    #1,address_flag
        bra     odd10

set_normal_address
	lsld
        clr     address_flag

odd10
	tst	s_record_type			;check if S2
	bne	LO_HALF

	cpy	#$c000				;if address is in last page
	bhs	yes_lastPage

	tst	BANKING
	bne	LO_HALF				;This mode only works for S1 type records in bank mode

	movb	LAST_PPAGE,PPAGE		;$C000 to $FFFF is last page
	dec	PPAGE				;decrement PPAGE so that range is $8000 to $BFFF

	bra	LO_HALF

yes_lastPage
	xgdy
	subd	#$4000				;Subtract $4000 to bring address to between $8000 to $BFFF
	xgdy

	movb	LAST_PPAGE,PPAGE		;$C000 to $FFFF is last page


LO_HALF						;Reg Y contains the address
        brclr   address_flag,%01,even_address

        dey					;Decrement to make Address even
        ldaa    0,y				;Get byte for the odd address
        psha

        bra     odd_address

even_address					;The address is even here
	jsr	GetHexByte			;get a byte (in A)
        psha					;Save high byte
        dec     byte_counter
	bne	odd_address			;Get byte for the odd address

	cpy	#FLASH				;if address below Flash then
	bhs	L2C
	pula
        staa    0,y				;do normal write (it's RAM or register)

	lbra	Get_S

L2C	
	ldab	#$ff			;Force lower byte $FF if address end as odd and no forth coming data
	pula
	bra	L1C
		
odd_address					;The address is odd here
        jsr     GetHexByte			;Get byte for the odd address
        dec     byte_counter
	
        tab					;transfer to lower byte
        pula					;Get high byte

        cpy	#FLASH				;if address below Flash then
	bhs	L1C

write10
	cpy	#$C00
	blo	check_ram
	bra	chk10

check_ram
	cpy	#$800
	bhs	write_temp

chk10
        std     0,y				;do normal write (it's RAM or register)
	bra	VF

write_temp
	std	temp				;do normal write to memory
	bra	VerfOK

L1C
	tst	WRITE_MODE			;else if not Verify then
	beq	VF

	jsr	WriteFlash			;do Flash write
        ldd     BYTE

	bra	VF

VF
        cpd     0,y				;if byte doesn't verify then
	beq	VerfOK	

	bset	PORTT,LED_ON			;turn on PT6 LED
	movb	#$ff,ERROR			;set error flag

	tst	first_err			;test if error already set
	bne	VerfOK

	movb	#$ff,first_err
	sty	abs_address
	movb	PPAGE,page_address	
	movw	0,y,bad_data			;save bad data
	std	good_data			;save good data

VerfOK
        iny					;The address should be even at this point
        iny

        ldab    byte_counter
        lbne    even_address
        lbra    Get_S				;get next S-record

PSX
	rts


;-----------------------------------------------------------------------------
WriteFlash 
	movb	PPAGE,CURPAGE			;preserve PPAGE value
        clr     PPAGE				;Flash needs to see address of $8554,
						;which is in second 16K page of Flash
        movw    #$AAAA,$8AAA    		;need offset of $8000 to generate CSP0*
        movw    #$5555,$8554    		;
        movw    #$A0A0,$8AAA    		;

	movb	CURPAGE,PPAGE			;restore PPAGE value
        std     BYTE     			;save for compare
        movw    BYTE,0,y        		;write Flash


DataPoll         
        ldd     BYTE             
	staa	TEMP1
	stab	TEMP2

DP1
	ldd	0,y				;get value being programmed
	eorb	TEMP2
	bpl	DPP				;until DQ7 matches data bit7 or

CheckDQ5 
        rolb
        rolb
        rolb
        bcc     DataPoll

	movw	#$1000,CNT2
	movw	#$ffff,CNT1

DPP	
	ldd	0,y				;verify data
	cpd	BYTE
	beq	DPX

	dec	CNT1
	bne	DPP
	dec	CNT2
	beq	Err
	ldd	BYTE
	bra	WriteFlash

Err
	movb	#$40,PORTT			;turn on PT6 LED
	movb    #$ff,ERROR			;set error flag

DPX 
	rts
